#include <inttypes.h>
#include <iostream>
#include <fstream>
#include <math.h>
#include "Hashtable.h"
#include "Charactertools.h"

using namespace std;

int main(int argc, char *argv[]) {
	Hashtable ht;
	Charactertools ct;

	streampos size;
	char * memblock;
	string ret_str;
	int ngram_size = 3;

	string input_file, output_file;

	cout << "********Massive NGram counter********" << endl;

	cout << argv[1] << " " << argv[2] << " " << argv[3] << " " << argv[4] << " ";
	if (argc != 5) {
		cout << "Wrong Arguments." << endl << "Usage:" << endl << "ngram [w/c word or character] [N] [input file] [output file]" << endl;
		cout << "Input file needs to be one byte encoded ISO8592" << endl;
		return(1);
	}

	cout << "Start..." << endl;

	ht.setLoadFactor(10.0);
	uint64_t chunk_size = 100000000;
	uint64_t mem_size = chunk_size;

	ngram_size = atoi(argv[2]);

	ifstream file(argv[3], ios::in | ios::binary | ios::ate);
	if (file.is_open())
	{
		int rounds = 1;
		size = file.tellg();
		uint64_t size_int = (uint64_t)size;
		cout << "Filesize: " << size_int / 1000000.0 << " MB" << endl;
		//Check if file is larger 100MB if yes split in multiple chunks
		if (size_int > chunk_size) {
			rounds = size_int / chunk_size + 1;
			memblock = new char[chunk_size];
		}
		else {
			mem_size = size_int;
			memblock = new char[size_int];
		}
		for (int k = 0; k < rounds; k++) {
			cout << endl << "Loading chunk into memory " << k + 1 << "/" << rounds << endl;
			if (k == rounds - 1) {
				mem_size = size_int - k*chunk_size;
			}
			file.seekg(k*chunk_size, ios::beg);
			file.read(memblock, mem_size);
			//******
			uint64_t i = 0;
			//Character counting
			if (argv[1][0] == 'c') {
				for (i = 0; i < mem_size - ngram_size; i++) {
					if (i % 10000000 == 0)
						cout << ".";
					ct.extract_ngram(memblock, i, ngram_size, ret_str);
					ht.plus_one(ret_str);
				}
			}
			//Word counting
			else if (argv[1][0] == 'w') {
				uint64_t new_pos;
				if (ngram_size == 1) {
					while (i < mem_size - 25) {
						if (i % 10000000 < 5)
							cout << ".";
						//Extract word and count one up
						new_pos = ct.extract_word(memblock, i, ret_str);
						if (new_pos != 0) {
							i = new_pos;
							ht.plus_one(ret_str);
						}
						else {
							i++;
						}
					}
				}
				else {
					while (i < mem_size - ngram_size * 25) {
						//Extract word and count one up
						new_pos = ct.extract_multiple_words(memblock, i, ngram_size, ret_str);
						if (new_pos != 0) {
							i = new_pos;
							ht.plus_one(ret_str);
						}
						else {
							i++;
						}
					}
				}
			}
			//******
		}

		file.close();

		ht.export_table(argv[4]);
		delete[] memblock;
	}

	cout << endl << "Done." << endl;
}
